package me.goldze.mvvmhabit.base;

import android.Manifest;
import android.app.ActivityManager;
import android.app.DatePickerDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.widget.DatePicker;

import com.afollestad.materialdialogs.MaterialDialog;
import com.huantansheng.easyphotos.EasyPhotos;
import com.huantansheng.easyphotos.callback.SelectCallback;
import com.tbruyelle.rxpermissions3.RxPermissions;
import com.trello.rxlifecycle4.components.support.RxAppCompatActivity;

import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Map;

import androidx.annotation.Nullable;
import androidx.databinding.DataBindingUtil;
import androidx.databinding.ViewDataBinding;
import androidx.fragment.app.FragmentActivity;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModel;
import androidx.lifecycle.ViewModelProviders;

import io.reactivex.rxjava3.annotations.NonNull;
import io.reactivex.rxjava3.disposables.Disposable;
import me.goldze.mvvmhabit.base.BaseViewModel.ParameterField;
import me.goldze.mvvmhabit.bus.Messenger;

import com.xx.xxviewlibrary.comm.CommTextAgreeDialog;
import com.xx.xxviewlibrary.comm.CommTextDialog;
import com.xx.xxviewlibrary.comm.NUllDialog;
import com.xx.xxviewlibrary.comm.RefuseDialogBean;
import com.xx.xxviewlibrary.utils.GlideEngine;

import me.goldze.mvvmhabit.utils.MaterialDialogUtils;
import me.goldze.mvvmhabit.utils.StringUtils;
import me.goldze.mvvmhabit.utils.ToastUtils;


/**
 * Created by goldze on 2017/6/15.
 * 一个拥有DataBinding框架的基Activity
 * 这里根据项目业务可以换成你自己熟悉的BaseActivity, 但是需要继承RxAppCompatActivity,方便LifecycleProvider管理生命周期
 */
public abstract class BaseActivity<V extends ViewDataBinding, VM extends BaseViewModel> extends RxAppCompatActivity implements IBaseView {
    protected V binding;
    protected VM viewModel;
    private int viewModelId;
    private MaterialDialog dialog;

    private NUllDialog nullDialog;
   public  RxPermissions rp;
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
       rp =new RxPermissions(this);
        //页面接受的参数方法
        initParam();
        //私有的初始化Databinding和ViewModel方法
        initViewDataBinding(savedInstanceState);
        //私有的ViewModel与View的契约事件回调逻辑
        registorUIChangeLiveDataCallBack();
        registorUIChangeLiveDataCallBackNoTouch();
        //页面数据初始化方法
        initData();
        //页面事件监听的方法，一般用于ViewModel层转到View层的事件注册
        initViewObservable();
        //注册RxBus
        viewModel.registerRxBus();
        initReceiver();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        //解除Messenger注册
        Messenger.getDefault().unregister(viewModel);
        if (viewModel != null) {
            viewModel.removeRxBus();
        }
        if (binding != null) {
            binding.unbind();
        }
    }

    /**
     * 注入绑定
     */
    private void initViewDataBinding(Bundle savedInstanceState) {
        //DataBindingUtil类需要在project的build中配置 dataBinding {enabled true }, 同步后会自动关联android.databinding包
        binding = DataBindingUtil.setContentView(this, initContentView(savedInstanceState));
        viewModelId = initVariableId();
        viewModel = initViewModel();
        if (viewModel == null) {
            Class modelClass;
            Type type = getClass().getGenericSuperclass();
            if (type instanceof ParameterizedType) {
                modelClass = (Class) ((ParameterizedType) type).getActualTypeArguments()[1];
            } else {
                //如果没有指定泛型参数，则默认使用BaseViewModel
                modelClass = BaseViewModel.class;
            }
            viewModel = (VM) createViewModel(this, modelClass);
        }
        //关联ViewModel
        binding.setVariable(viewModelId, viewModel);
        //支持LiveData绑定xml，数据改变，UI自动会更新
        binding.setLifecycleOwner(this);
        //让ViewModel拥有View的生命周期感应
        getLifecycle().addObserver(viewModel);
        //注入RxLifecycle生命周期
        viewModel.injectLifecycleProvider(this);
    }

    //刷新布局
    public void refreshLayout() {
        if (viewModel != null) {
            binding.setVariable(viewModelId, viewModel);
        }
    }


    /**
     * =====================================================================
     **/
    //注册ViewModel与View的契约UI回调事件
    protected void registorUIChangeLiveDataCallBack() {
        //加载对话框显示
        viewModel.getUC().getShowDialogEventNoTouch().observe(this, new Observer<String>() {
            @Override
            public void onChanged(@Nullable String title) {
                showNoTouchDialog(title);
            }
        });
        //加载对话框消失
        viewModel.getUC().getDismissDialogEvent().observe(this, new Observer<Void>() {
            @Override
            public void onChanged(@Nullable Void v) {
                dismissDialog();
            }
        });
        //跳入新页面
        viewModel.getUC().getStartActivityEvent().observe(this, new Observer<Map<String, Object>>() {
            @Override
            public void onChanged(@Nullable Map<String, Object> params) {
                Bundle bundle = (Bundle) params.get(ParameterField.BUNDLE);
                if (params.get(ParameterField.CLASS_TYPE) == null) {
                    Class<?> clz = (Class<?>) params.get(ParameterField.CLASS);
                    startActivity(clz, bundle);
                } else if (ParameterField.Result_TYPE.equals(params.get(ParameterField.CLASS_TYPE))) {

                    Intent intent = new Intent();
                    intent.putExtras(bundle);
                    setResult(RESULT_OK, intent);
                    finish();
                } else {
                    Class<?> clz = (Class<?>) params.get(ParameterField.CLASS);
                    Intent intent = new Intent(BaseActivity.this, clz);
                    if (bundle != null) {
                        intent.putExtras(bundle);
                    }
                    startActivityForResult(intent, (Integer) params.get(ParameterField.REQUEST_TYPE));
                }

            }
        });
        //跳入ContainerActivity
        viewModel.getUC().getStartContainerActivityEvent().observe(this, new Observer<Map<String, Object>>() {
            @Override
            public void onChanged(@Nullable Map<String, Object> params) {
                String canonicalName = (String) params.get(ParameterField.CANONICAL_NAME);
                Bundle bundle = (Bundle) params.get(ParameterField.BUNDLE);
                startContainerActivity(canonicalName, bundle);
            }
        });
        //关闭界面
        viewModel.getUC().getFinishEvent().observe(this, new Observer<Void>() {
            @Override
            public void onChanged(@Nullable Void v) {
                finish();
            }
        });
        //关闭上一层
        viewModel.getUC().getOnBackPressedEvent().observe(this, new Observer<Void>() {
            @Override
            public void onChanged(@Nullable Void v) {
                onBackPressed();
            }
        });

        viewModel.getUC().getStartPhotoEvent().observe(this, new Observer<Map<String, Object>>() {
            @Override
            public void onChanged(Map<String, Object> o) {


               if ( !rp.isGranted(Manifest.permission.CAMERA) || !rp.isGranted(Manifest.permission.READ_EXTERNAL_STORAGE))
                {


                        showCommTextDialog(new RefuseDialogBean(o.get("pMsg").toString() + ",是否同意", "同意", "拒绝"), new CommTextDialog.onAcceptCallBack() {
                            @Override
                            public void acceptBack() {
                                rp.request(Manifest.permission.CAMERA,Manifest.permission.READ_EXTERNAL_STORAGE).subscribe(new io.reactivex.rxjava3.core.Observer<Boolean>() {
                                    @Override
                                    public void onSubscribe(@NonNull Disposable d) {

                                    }

                                    @Override
                                    public void onNext(@NonNull Boolean aBoolean) {

                                        if (aBoolean)
                                        {
                                            boolean hasCamera = (boolean) o.get("hasCamera");
                                            int maxCount = (int) o.get("maxCount");
                                            if (o.get("maxCount") == null) {
                                                maxCount = 1;
                                            }
                                            SelectCallback callback = (SelectCallback) o.get("callBack");
                                            startPhoto(hasCamera, maxCount, callback);
                                        }else {
                                            ToastUtils.showShort("请开启拍照，和文件读取权限，否则无法正常使用完整功能");
                                        }
                                    }

                                    @Override
                                    public void onError(@NonNull Throwable e) {
                                        ToastUtils.showShort("请开启拍照，和文件读取权限，否则无法正常使用完整功能");
                                    }

                                    @Override
                                    public void onComplete() {

                                    }
                                });
                            }

                            @Override
                            public void refuseBack() {

                            }
                        });
                }else {
                   rp.request(Manifest.permission.CAMERA,Manifest.permission.READ_EXTERNAL_STORAGE).subscribe(new io.reactivex.rxjava3.core.Observer<Boolean>() {
                       @Override
                       public void onSubscribe(@NonNull Disposable d) {

                       }

                       @Override
                       public void onNext(@NonNull Boolean aBoolean) {

                           if (aBoolean)
                           {
                               boolean hasCamera = (boolean) o.get("hasCamera");
                               int maxCount = (int) o.get("maxCount");
                               if (o.get("maxCount") == null) {
                                   maxCount = 1;
                               }
                               SelectCallback callback = (SelectCallback) o.get("callBack");
                               startPhoto(hasCamera, maxCount, callback);
                           }else {
                               ToastUtils.showShort("请开启拍照，和文件读取权限，否则无法正常使用完整功能");
                           }
                       }

                       @Override
                       public void onError(@NonNull Throwable e) {

                       }

                       @Override
                       public void onComplete() {

                       }
                   });
               }



            }
        });

        //调用日期选择弹窗
        viewModel.getUC().getShowDataDialogEvent().observe(this, new Observer<Map<String, Object>>() {

            @Override
            public void onChanged(Map<String, Object> stringObjectMap) {
                int year = (int) stringObjectMap.get("year");
                int month = (int) stringObjectMap.get("month");
                int day = (int) stringObjectMap.get("day");
                onDataSelect ondata = (onDataSelect) stringObjectMap.get("ondata");
                showDataDialog(year, month, day, ondata);
            }
        });

        viewModel.getUC().getShowTextDialogEvent().observe(this, new Observer<Map<String, Object>>() {
            @Override
            public void onChanged(Map<String, Object> o) {
                RefuseDialogBean bean = (RefuseDialogBean) o.get("bean");
                CommTextDialog.onAcceptCallBack callBack = (CommTextDialog.onAcceptCallBack) o.get("callBack");
                showCommTextDialog(bean, callBack);
            }
        });
    }

    /**
     * 触发禁用返回键
     */
    public void showNoTouchDialog(String title) {
        if (StringUtils.isEmpty(title)) {
            if (nullDialog != null) {
                nullDialog.show();
            } else {
                nullDialog = new NUllDialog(this);
                nullDialog.show();
            }
        } else {
            if (dialog != null) {
                dialog = dialog.getBuilder().title(title).build();
                dialog.show();
            } else {
                MaterialDialog.Builder builder = MaterialDialogUtils.showIndeterminateProgressNoTouchDialog(this, title, true);
                dialog = builder.show();
            }
        }
    }

        /**
         * =====================================================================禁止返回
         **/
    //注册ViewModel与View的契约UI回调事件
    protected void registorUIChangeLiveDataCallBackNoTouch() {
        //加载对话框消失
        viewModel.getUC().getDismissDialogEvent().observe(this, new Observer<Void>() {
            @Override
            public void onChanged(@Nullable Void v) {
                dismissDialog();
            }
        });
        //跳入新页面
        viewModel.getUC().getStartActivityEvent().observe(this, new Observer<Map<String, Object>>() {
            @Override
            public void onChanged(@Nullable Map<String, Object> params) {
                Bundle bundle = (Bundle) params.get(ParameterField.BUNDLE);
                if ( params.get(ParameterField.CLASS_TYPE)==null)
                {
                    Class<?> clz = (Class<?>) params.get(ParameterField.CLASS);
                    startActivity(clz, bundle);
                }else if (ParameterField.Result_TYPE.equals(params.get(ParameterField.CLASS_TYPE))){

                    Intent intent=new Intent();
                    intent.putExtras(bundle);
                    setResult( RESULT_OK,intent);
                    finish();
                }else {
                    Class<?> clz = (Class<?>) params.get(ParameterField.CLASS);
                    Intent intent = new Intent(BaseActivity.this, clz);
                    if (bundle != null) {
                        intent.putExtras(bundle);
                    }
                    startActivityForResult(intent, (Integer) params.get(ParameterField.REQUEST_TYPE));
                }

            }
        });
        //跳入ContainerActivity
        viewModel.getUC().getStartContainerActivityEvent().observe(this, new Observer<Map<String, Object>>() {
            @Override
            public void onChanged(@Nullable Map<String, Object> params) {
                String canonicalName = (String) params.get(ParameterField.CANONICAL_NAME);
                Bundle bundle = (Bundle) params.get(ParameterField.BUNDLE);
                startContainerActivity(canonicalName, bundle);
            }
        });
        //关闭界面
        viewModel.getUC().getFinishEvent().observe(this, new Observer<Void>() {
            @Override
            public void onChanged(@Nullable Void v) {
                finish();
            }
        });
        //关闭上一层
        viewModel.getUC().getOnBackPressedEvent().observe(this, new Observer<Void>() {
            @Override
            public void onChanged(@Nullable Void v) {
                onBackPressed();
            }
        });

        viewModel.getUC().getStartPhotoEvent().observe(this, new Observer<Map<String, Object>>() {
            @Override
            public void onChanged(Map<String, Object> o) {
                boolean hasCamera= (boolean) o.get("hasCamera");
                int maxCount= (int) o.get("maxCount");
                if(o.get("maxCount")==null)
                {
                    maxCount=1;
                }
                SelectCallback callback= (SelectCallback) o.get("callBack");
                startPhoto(hasCamera,maxCount,callback);
            }
        });

        //调用日期选择弹窗
        viewModel.getUC().getShowDataDialogEvent().observe(this,new Observer<Map<String, Object>>()
        {

            @Override
            public void onChanged(Map<String, Object> stringObjectMap) {
                int year= (int) stringObjectMap.get("year");
                int month= (int) stringObjectMap.get("month");
                int day= (int) stringObjectMap.get("day");
                onDataSelect ondata= (onDataSelect) stringObjectMap.get("ondata");
                showDataDialog(year,month,day,ondata);
            }
        });

        viewModel.getUC().getShowTextDialogEvent().observe(this, new Observer<Map<String, Object>>() {
            @Override
            public void onChanged(Map<String, Object> o) {
                RefuseDialogBean bean= (RefuseDialogBean) o.get("bean");
                CommTextDialog.onAcceptCallBack callBack= (CommTextDialog.onAcceptCallBack) o.get("callBack");
                showCommTextDialog(bean,callBack);
            }
        });
    }

    /**
     * 选择图片
     */
    public void startPhoto(boolean hasCamera,SelectCallback callback)
    {
        startPhoto(hasCamera,1,callback);
    }

    public void startPhoto(boolean hasCamera,int count,SelectCallback callback)
    {


        EasyPhotos.createAlbum(this, hasCamera, false, GlideEngine.getInstance()).setCount(count)
                .setFileProviderAuthority(getAppProcessName(this) + ".fileProvider")
                .start(callback);
    }

    public void showDialog(String title) {
        if (StringUtils.isEmpty(title)){
            if (nullDialog!=null)
            {
                nullDialog.show();
            }else {
                nullDialog=new NUllDialog(this);
                nullDialog.show();
            }
        }else {
            if (dialog != null) {
                dialog = dialog.getBuilder().title(title).build();
                dialog.show();
            } else {
                MaterialDialog.Builder builder = MaterialDialogUtils.showIndeterminateProgressDialog(this, title, true);
                dialog = builder.show();
            }
        }


    }




    public void showCommTextDialog(RefuseDialogBean bean,CommTextDialog.onAcceptCallBack callBack)
    {
        CommTextDialog textDialog=new CommTextDialog(this,bean,callBack);
        textDialog.show();
    }
    public void showCommTextAgreeDialog(RefuseDialogBean bean, CommTextAgreeDialog.onAcceptCallBack callBack) {
        CommTextAgreeDialog textDialog = new CommTextAgreeDialog(this, bean, callBack);
        textDialog.show();
    }


    public void dismissDialog() {
        if (dialog != null && dialog.isShowing()) {
            dialog.dismiss();
        }
        if (nullDialog != null && nullDialog.isShowing()) {
            nullDialog.dismiss();
        }
    }

    /**
     * 跳转页面
     *
     * @param clz 所跳转的目的Activity类
     */
    public void startActivity(Class<?> clz) {
        startActivity(new Intent(this, clz));
    }

    /**
     * 跳转页面
     *
     * @param clz    所跳转的目的Activity类
     * @param bundle 跳转所携带的信息
     */
    public void startActivity(Class<?> clz, Bundle bundle) {
        Intent intent = new Intent(this, clz);
        if (bundle != null) {
            intent.putExtras(bundle);
        }
        startActivity(intent);
    }

    /**
     * 跳转容器页面
     *
     * @param canonicalName 规范名 : Fragment.class.getCanonicalName()
     */
    public void startContainerActivity(String canonicalName) {
        startContainerActivity(canonicalName, null);
    }

    /**
     * 跳转容器页面
     *
     * @param canonicalName 规范名 : Fragment.class.getCanonicalName()
     * @param bundle        跳转所携带的信息
     */
    public void startContainerActivity(String canonicalName, Bundle bundle) {
        Intent intent = new Intent(this, ContainerActivity.class);
        intent.putExtra(ContainerActivity.FRAGMENT, canonicalName);
        if (bundle != null) {
            intent.putExtra(ContainerActivity.BUNDLE, bundle);
        }
        startActivity(intent);
    }

    /**
     * 展示日期选择
     * @param year
     * @param month
     * @param day
     * @param onData
     */
    public void showDataDialog(int year,int  month,int day,onDataSelect onData)
    {

        DatePickerDialog datePickerDialog = new DatePickerDialog(this, new DatePickerDialog.OnDateSetListener() {

            @Override
            public void onDateSet(DatePicker view, int year, int month, int dayOfMonth) {
                try {
                    SimpleDateFormat df=new SimpleDateFormat("yyyy-MM-dd");
                    Date date= df.parse(year + "-" + (month + 1) + "-" + dayOfMonth);
                    onData.reData(df.format(date) );
                } catch (ParseException e) {
                    onData.reData( "");
                    throw new RuntimeException(e);
                }


            }
        }, year, month, day) ;

        datePickerDialog.setMessage("日期选择");
        datePickerDialog.show();
    }

    /**
     * =====================================================================
     **/
    @Override
    public void initParam() {

    }

    /**
     * 初始化根布局
     *
     * @return 布局layout的id
     */
    public abstract int initContentView(Bundle savedInstanceState);

    /**
     * 初始化ViewModel的id
     *
     * @return BR的id
     */
    public abstract int initVariableId();

    /**
     * 初始化ViewModel
     *
     * @return 继承BaseViewModel的ViewModel
     */
    public VM initViewModel() {
        return null;
    }

    @Override
    public void initData() {

    }

    @Override
    public void initViewObservable() {

    }

    private void initReceiver() {
        IntentFilter intentFilter = new IntentFilter();
        intentFilter.addAction("CLOSE_ALL");
        BroadcastReceiver broadcastReceiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                if (!getClass().getSimpleName().equals("LoginActivity")) {
                    finish();
                }
            }
        };
        registerReceiver(broadcastReceiver, intentFilter);
    }

    /**
     * 创建ViewModel
     *
     * @param cls
     * @param <T>
     * @return
     */
    public <T extends ViewModel> T createViewModel(FragmentActivity activity, Class<T> cls) {
        return ViewModelProviders.of(activity).get(cls);
    }


    public   int getApp(Context context) {


        switch (getAppProcessName(context)) {
            case "com.xx.merchanthbh":
                return 0;

            case "com.xx.hbhconsumer":

                return 1;
            case "com.xx.hbhbcompany":

                return 2;
            default:
                return 0;
        }

    }

    /**
     * 获取当前应用程序的包名
     *
     * @param context 上下文对象
     * @return 返回包名
     */
    public  String getAppProcessName(Context context) {
        //当前应用pid
        int pid = android.os.Process.myPid();
        //任务管理类
        ActivityManager manager = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
        //遍历所有应用
        List<ActivityManager.RunningAppProcessInfo> infos = manager.getRunningAppProcesses();
        for (ActivityManager.RunningAppProcessInfo info : infos) {
            if (info.pid == pid)//得到当前应用
                return info.processName;//返回包名
        }
        return "";
    }
}
