package me.goldze.mvvmhabit.base;

import android.app.Application;
import android.os.Bundle;
import android.view.View;

import java.lang.ref.WeakReference;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;

import androidx.annotation.NonNull;
import androidx.lifecycle.AndroidViewModel;
import androidx.lifecycle.Lifecycle;
import androidx.lifecycle.LifecycleOwner;
import androidx.lifecycle.Observer;

import com.huantansheng.easyphotos.callback.SelectCallback;
import com.trello.rxlifecycle4.LifecycleProvider;
import com.xx.xxviewlibrary.comm.CommTextDialog;
import com.xx.xxviewlibrary.comm.RefuseDialogBean;

import io.reactivex.rxjava3.disposables.CompositeDisposable;
import io.reactivex.rxjava3.disposables.Disposable;
import io.reactivex.rxjava3.functions.Consumer;
import me.goldze.mvvmhabit.bus.event.SingleLiveEvent;

/**
 * Created by goldze on 2017/6/15.
 */
public class BaseViewModel<M extends BaseModel> extends AndroidViewModel implements IBaseViewModel, Consumer<Disposable> {
    protected M model;
    private UIChangeLiveData uc;
    //弱引用持有
    private WeakReference<LifecycleProvider> lifecycle;
    //管理RxJava，主要针对RxJava异步操作造成的内存泄漏
    private CompositeDisposable mCompositeDisposable;

    public BaseViewModel(@NonNull Application application) {
        this(application, null);
    }

    public BaseViewModel(@NonNull Application application, M model) {
        super(application);
        this.model = model;
        mCompositeDisposable = new CompositeDisposable();
    }



    protected void addSubscribe(Disposable disposable) {
        if (mCompositeDisposable == null) {
            mCompositeDisposable = new CompositeDisposable();
        }
        mCompositeDisposable.add(disposable);
    }

    /**
     * 注入RxLifecycle生命周期
     *
     * @param lifecycle
     */
    public void injectLifecycleProvider(LifecycleProvider lifecycle) {
        this.lifecycle = new WeakReference<>(lifecycle);
    }

    public LifecycleProvider getLifecycleProvider() {
        return lifecycle.get();
    }

    public UIChangeLiveData getUC() {
        if (uc == null) {
            uc = new UIChangeLiveData();
        }
        return uc;
    }

    public void showDialog() {
        showDialog("请稍后...");
    }

    public void showDialog(String title) {

     uc.showDialogEventNoTouch.postValue(title);
        /*更新版本下载中的时候禁用返回上一步*/
        uc.showDialogEventNoTouch.postValue(title);
    }

    public void dismissDialog() {
        uc.dismissDialogEvent.call();
    }

    /**
     * 跳转页面
     *
     * @param clz 所跳转的目的Activity类
     */
    public void startActivity(Class<?> clz) {
        startActivity(clz, null);
    }

    /**
     * 跳转页面
     *setResult返回
     * @param bundle 跳转所携带的信息
     */
    public void startActivity(int resultCode, Bundle bundle) {
        Map<String, Object> params = new HashMap<>();
        params.put(ParameterField.CLASS_TYPE, ParameterField.Result_TYPE);
        params.put(ParameterField.Result_TYPE, resultCode);

        if (bundle != null) {
            params.put(ParameterField.BUNDLE, bundle);
        }
        uc.startActivityEvent.postValue(params);
    }

    /**
     * 跳转页面
     *
     * @param clz    所跳转的目的Activity类
     * @param bundle 跳转所携带的信息
     */
    public void startResultActivity(Class<?> clz, int requestCode,Bundle bundle) {
        Map<String, Object> params = new HashMap<>();
        params.put(ParameterField.CLASS_TYPE, ParameterField.REQUEST_TYPE);
        params.put(ParameterField.REQUEST_TYPE, requestCode);
        params.put(ParameterField.CLASS, clz);
        if (bundle != null) {
            params.put(ParameterField.BUNDLE, bundle);
        }
        uc.startActivityEvent.postValue(params);
    }

    /**
     * 跳转页面
     *
     * @param clz    所跳转的目的Activity类
     * @param bundle 跳转所携带的信息
     */
    public void startActivity(Class<?> clz, Bundle bundle) {
        Map<String, Object> params = new HashMap<>();
        params.put(ParameterField.CLASS, clz);
        if (bundle != null) {
            params.put(ParameterField.BUNDLE, bundle);
        }
        uc.startActivityEvent.postValue(params);
    }

    /**
     * 跳转容器页面
     *
     * @param canonicalName 规范名 : Fragment.class.getCanonicalName()
     */
    public void startContainerActivity(String canonicalName) {
        startContainerActivity(canonicalName, null);
    }

    /**
     * 跳转容器页面
     *
     * @param canonicalName 规范名 : Fragment.class.getCanonicalName()
     * @param bundle        跳转所携带的信息
     */
    public void startContainerActivity(String canonicalName, Bundle bundle) {
        Map<String, Object> params = new HashMap<>();
        params.put(ParameterField.CANONICAL_NAME, canonicalName);
        if (bundle != null) {
            params.put(ParameterField.BUNDLE, bundle);
        }
        uc.startContainerActivityEvent.postValue(params);
    }

    /**
     * 关闭界面
     */
    public void finish() {
        uc.finishEvent.call();
    }

    /**
     * 返回上一层
     */
    public void onBackPressed() {
        uc.onBackPressedEvent.call();
    }

    /**
     * 获取照片，图片
     * @param hasCamera 是否有照相机功能
     * @param callback
     */
    public void startPhotoPage(boolean hasCamera, String msg,SelectCallback callback)
    {
        Map<String, Object> photomap = new HashMap<>();
        photomap.put("hasCamera", hasCamera);
        photomap.put("maxCount", 1);
        photomap.put("pMsg",msg);

        if (callback != null) {
            photomap.put("callBack", callback);
        }
        uc.startPhotoEvent.postValue(photomap);
    }

    /**
     * 获取图片
     * @param hasCamera 是否有照相机功能
     * @param maxCount 选则最大数量
     * @param callback
     */
    public void startPhotoPage(boolean hasCamera,int maxCount, String msg,SelectCallback callback)
    {
        Map<String, Object> photomap = new HashMap<>();
        photomap.put("hasCamera", hasCamera);
        photomap.put("maxCount", maxCount);
        photomap.put("pMsg",msg);
        if (callback != null) {
            photomap.put("callBack", callback);
        }
        uc.startPhotoEvent.postValue(photomap);
    }

    /**
     * 展示text弹窗
     * @param bean
     * @param callBack
     */
    public void showCommTextDialog(RefuseDialogBean bean, CommTextDialog.onAcceptCallBack callBack)
    {
        Map<String, Object> textDialogMap = new HashMap<>();
        textDialogMap.put("bean", bean);

        if (callBack != null) {
            textDialogMap.put("callBack", callBack);
        }
        uc.showTextDialogEvent.postValue(textDialogMap);
    }


    /**
     * 选择日期
     * @param calendar
     * @param ondata
     */
    public void showDataDialog(Calendar calendar, onDataSelect ondata)
    {
        int year = calendar.get(Calendar.YEAR);
        int month = calendar.get(Calendar.MONTH);
        int day = calendar.get(Calendar.DAY_OF_MONTH);
        showDataDialog(year,month,day,ondata);
    }

    /**
     * 选择日期
     * @param year
     * @param month
     * @param day
     * @param ondata
     */
    public void showDataDialog(int year,int  month,int day, onDataSelect ondata)
    {
        Map<String, Object> dataMap = new HashMap<>();
        dataMap.put("year", year);
        dataMap.put("month", month);
        dataMap.put("day", day);
        if (ondata != null) {
            dataMap.put("ondata", ondata);
        }
        uc.showDataDialogEvent.postValue(dataMap);
    }


    public View.OnClickListener clickFinish=new View.OnClickListener() {
        @Override
        public void onClick(View view) {
            finish();
        }
    };







    @Override
    public void onAny(LifecycleOwner owner, Lifecycle.Event event) {
    }

    @Override
    public void onCreate() {
    }

    @Override
    public void onDestroy() {
    }

    @Override
    public void onStart() {
    }

    @Override
    public void onStop() {
    }

    @Override
    public void onResume() {
    }

    @Override
    public void onPause() {
    }

    @Override
    public void registerRxBus() {
    }

    @Override
    public void removeRxBus() {
    }

    @Override
    protected void onCleared() {
        super.onCleared();
        if (model != null) {
            model.onCleared();
        }
        //ViewModel销毁时会执行，同时取消所有异步任务
        if (mCompositeDisposable != null) {
            mCompositeDisposable.clear();
        }
    }

    @Override
    public void accept(Disposable disposable) throws Exception {
        addSubscribe(disposable);
    }

    public final class UIChangeLiveData extends SingleLiveEvent {
        private SingleLiveEvent<String> showDialogEvent;

        /*更新版本下载中的时候禁用返回上一步*/
        private SingleLiveEvent<String> showDialogEventNoTouch;

        private SingleLiveEvent<Void> dismissDialogEvent;
        private SingleLiveEvent<Map<String, Object>> startActivityEvent;
        private SingleLiveEvent<Map<String, Object>> startContainerActivityEvent;
        private SingleLiveEvent<Void> finishEvent;
        private SingleLiveEvent<Void> onBackPressedEvent;

        private SingleLiveEvent<Map<String, Object>> startPhotoEvent;


        private SingleLiveEvent<Map<String, Object>> showDataDialogEvent;

        private SingleLiveEvent<Map<String,Object>> showTextDialogEvent;




        public SingleLiveEvent<String> getShowDialogEvent() {
            return showDialogEvent = createLiveData(showDialogEvent);
        }

        /*更新版本下载中的时候禁用返回上一步*/
        public SingleLiveEvent<String> getShowDialogEventNoTouch() {
            return showDialogEventNoTouch = createLiveData(showDialogEventNoTouch);
        }

        public SingleLiveEvent<Void> getDismissDialogEvent() {
            return dismissDialogEvent = createLiveData(dismissDialogEvent);
        }

        public SingleLiveEvent<Map<String, Object>> getStartActivityEvent() {
            return startActivityEvent = createLiveData(startActivityEvent);
        }
        public SingleLiveEvent<Map<String, Object>> getStartPhotoEvent()
        {
            return startPhotoEvent=createLiveData(startPhotoEvent);
        }
        public SingleLiveEvent<Map<String, Object>> getShowDataDialogEvent()
        {
            return showDataDialogEvent=createLiveData(showDataDialogEvent);
        }


        public SingleLiveEvent<Map<String, Object>> getShowTextDialogEvent()
        {
            return showTextDialogEvent=createLiveData(showTextDialogEvent);
        }

        public SingleLiveEvent<Map<String, Object>> getStartContainerActivityEvent() {
            return startContainerActivityEvent = createLiveData(startContainerActivityEvent);
        }

        public SingleLiveEvent<Void> getFinishEvent() {
            return finishEvent = createLiveData(finishEvent);
        }

        public SingleLiveEvent<Void> getOnBackPressedEvent() {
            return onBackPressedEvent = createLiveData(onBackPressedEvent);
        }

        private <T> SingleLiveEvent<T> createLiveData(SingleLiveEvent<T> liveData) {
            if (liveData == null) {
                liveData = new SingleLiveEvent<>();
            }
            return liveData;
        }

        @Override
        public void observe(LifecycleOwner owner, Observer observer) {
            super.observe(owner, observer);
        }
    }

    public static final class ParameterField {
        public static String CLASS = "CLASS";
        public static String CANONICAL_NAME = "CANONICAL_NAME";
        public static String BUNDLE = "BUNDLE";

        public static String CLASS_TYPE="CLASS_TYPE";
        public static String Result_TYPE="Result_TYPE";

        public static String REQUEST_TYPE="Request_TYPE";
    }
}
