package me.goldze.mvvmhabit.http;

import android.app.Activity;
import android.app.ActivityManager;
import android.content.Context;
import android.content.Intent;

import java.util.List;

import io.reactivex.rxjava3.observers.DisposableObserver;
import com.xx.xxviewlibrary.base.AppManager;
import me.goldze.mvvmhabit.utils.KLog;
import me.goldze.mvvmhabit.utils.ToastUtils;
import me.goldze.mvvmhabit.utils.Utils;

/**
 * Created by goldze on 2017/5/10.
 * 统一的Code封装处理。该类仅供参考，实际业务逻辑, 根据需求来定义，
 */

public abstract class ApiDisposablePageObserver<T> extends DisposableObserver<BasePageResponse<T>> {


    @Override
    public void onComplete() {

    }

    @Override
    public void onError(Throwable e) {
        e.printStackTrace();
        if (e instanceof ResponseThrowable) {
            ResponseThrowable rError = (ResponseThrowable) e;
            ToastUtils.showShort(rError.message);
            return;
        }
        //其他全部甩锅网络异常
        ToastUtils.showShort("网络异常");
    }

    @Override
    public void onStart() {
        super.onStart();
//        ToastUtils.showShort("http is start");
        // if  NetworkAvailable no !   must to call onCompleted
        if (!NetworkUtil.isNetworkAvailable(Utils.getContext())) {
            KLog.d("无网络，读取缓存数据");
            onComplete();
        }
    }

    @Override
    public void onNext(BasePageResponse<T> baseResponse) {
        switch (baseResponse.getCode()) {
            case CodeRule.CODE_200:
                //请求成功, 正确的操作方式
                onSuccess(baseResponse.getRows());
                getMsg(baseResponse.getMsg());
                getTotal(baseResponse.getTotal());
                break;
            case CodeRule.CODE_220:
                // 请求成功, 正确的操作方式, 并消息提示
                onSuccess(baseResponse.getRows());
                getTotal(baseResponse.getTotal());
                break;
            case CodeRule.CODE_300:
                //请求失败，不打印Message
                KLog.e("请求失败");
                ToastUtils.showShort("错误代码:", baseResponse.getCode());
                break;
            case CodeRule.CODE_330:
                //请求失败，打印Message
                ToastUtils.showShort(baseResponse.getMsg());
                break;
            case CodeRule.CODE_500:
                //服务器内部异常
                ToastUtils.showShort( baseResponse.getMsg());
                onError(baseResponse.getMsg());
                break;
            case CodeRule.CODE_503:
                //参数为空
                KLog.e("参数为空");
                break;
            case CodeRule.CODE_502:
                //没有数据
                KLog.e("没有数据");
                break;
            case CodeRule.CODE_510:
                //无效的Token，提示跳入登录页
                ToastUtils.showShort("token已过期，请重新登录");
                //关闭所有页面
                AppManager.getAppManager().finishAllActivity();
                //跳入登录界面
                //*****该类仅供参考，实际业务Code, 根据需求来定义，******//
                break;
            case CodeRule.CODE_530:
                ToastUtils.showShort("请先登录");
                break;
            case CodeRule.CODE_551:
                ToastUtils.showShort("错误代码:", baseResponse.getCode());
                break;
            case ApiDisposableObserver.CodeRule.CODE_401:
                ToastUtils.showShort("登陆失效，请重新登陆");
                Activity context= AppManager.getAppManager().currentActivity();
                Class casedetaiclass = null;
                try {
                    if (getApp(context) == 0) {
                        casedetaiclass = Class.forName("com.xx.merchanthbh.ui.login.LoginActivity");
                    }else if (getApp(context) == 1) {
                        casedetaiclass = Class.forName("com.xx.hbhconsumer.ui.login.LoginActivity");
                    }else {
                        casedetaiclass = Class.forName("com.xx.hbhbcompany.ui.login.LoginActivity");
                    }
                } catch (ClassNotFoundException e) {
                    e.printStackTrace();
                }

                Intent intent = new Intent(context, casedetaiclass);
                context.startActivity(intent);
                break;
            default:
                ToastUtils.showShort("错误代码:", baseResponse.getCode());
                break;
        }
    }

    public static final class CodeRule {
        //请求成功, 正确的操作方式
        static final int CODE_200 = 200;
        //请求成功, 消息提示
        static final int CODE_220 = 220;
        //请求失败，不打印Message
        static final int CODE_300 = 300;
        //请求失败，打印Message
        static final int CODE_330 = 330;
        //服务器内部异常
        static final int CODE_500 = 500;
        //参数为空
        static final int CODE_503 = 503;
        //没有数据
        static final int CODE_502 = 502;
        //无效的Token
        static final int CODE_510 = 510;
        //未登录
        static final int CODE_530 = 530;
        //请求的操作异常终止：未知的页面类型
        static final int CODE_551 = 551;
        //登陆失效
        static final int CODE_401 = 401;
    }

    //请求成功且返回码为200的回调方法,这里抽象方法申明
    public abstract void onSuccess(List<T> resultBean);

    //请求成功但返回的code码不是200的回调方法,这里抽象方法申明
    public abstract void onError(String Error);

    public void getMsg(String msg) {

    }

    public abstract void getTotal(int total) ;

    public   int getApp(Context context) {


        switch (getAppProcessName(context)) {
            case "com.xx.merchanthbh":
                return 0;

            case "com.xx.hbhconsumer":

                return 1;
            case "com.xx.hbhbcompany":

                return 2;
            default:
                return 0;
        }

    }

    /**
     * 获取当前应用程序的包名
     *
     * @param context 上下文对象
     * @return 返回包名
     */
    public  String getAppProcessName(Context context) {
        //当前应用pid
        int pid = android.os.Process.myPid();
        //任务管理类
        ActivityManager manager = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
        //遍历所有应用
        List<ActivityManager.RunningAppProcessInfo> infos = manager.getRunningAppProcesses();
        for (ActivityManager.RunningAppProcessInfo info : infos) {
            if (info.pid == pid)//得到当前应用
                return info.processName;//返回包名
        }
        return "";
    }
}